using AdventureClient;
using Log;
using Newtonsoft.Json.Linq;
using Protocols;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Web;
using Utility;

namespace LobbyServer
{

    class stCode
    {
        public int code;
    }
    public class JSONUtil
    {
        //JSON字符串压缩
        public static string JsonCompress(string json)
        {
            StringBuilder sb = new StringBuilder();
            using (StringReader reader = new StringReader(json))
            {
                int ch = -1;
                int lastch = -1;
                bool isQuoteStart = false;
                while ((ch = reader.Read()) > -1)
                {
                    if ((char)lastch != '\\' && (char)ch == '\"')
                    {
                        if (isQuoteStart)
                        {
                            isQuoteStart = true;
                        }
                        else
                        {
                            isQuoteStart = false;
                        }
                    }
                    if (!Char.IsWhiteSpace((char)ch) || isQuoteStart)
                    {
                        sb.Append((char)ch);
                    }
                    lastch = ch;
                }
            }
            return sb.ToString();
        }
    }

    /// <summary>
    /// hptt监听类
    /// -接收http消息并处理
    /// </summary>
    public class HttpListener
    {
        private const string LogTitle = "HttpResponse";

        /// <summary>
        /// 用户token值
        /// k=用户id v=token值
        /// </summary>
        private Dictionary<ulong, string> userTokens = new Dictionary<ulong, string>();

        //////////////////////////////////////////////////////////////////////////////////////////

        /// <summary>
        /// 初始化http返回
        /// </summary>
        public bool Init()
        {
            var cfg = Env.otherConfig.httpPostConfig;

            if (initListener(cfg.ListenPrefixes))
                return true;

            return false;
        }

        /// <summary>
        /// 初始化监听
        /// </summary>
        /// <param name="prefixes"></param>
        /// <returns></returns>
        private bool initListener(string prefixes)
        {
            Console.WriteLine(" in init..." + prefixes);

            if (!System.Net.HttpListener.IsSupported)
            {
                LogSys.Warn("Windows XP SP2 or Server 2003 is required to use the HttpListener class.");
                return false;
            }

            System.Net.HttpListener listener = new System.Net.HttpListener();
            listener.Prefixes.Add(prefixes);
            addAddress(prefixes, Environment.UserDomainName, Environment.UserName);
            listener.Start();//如果启动失败，以管理员身份运行

            startListen(listener);

            return true;
        }

        /// <summary>
        /// 添加监听地址
        /// </summary>
        /// <param name="address"></param>
        /// <param name="domain"></param>
        /// <param name="user"></param>
        private void addAddress(string address, string domain, string user)
        {
            string argsDll = String.Format(@"http delete urlacl url={0}", address);
            string args = string.Format(@"http add urlacl url={0} user={1}\{2}", address, domain, user);
            ProcessStartInfo psi = new ProcessStartInfo("netsh", argsDll);
            psi.Verb = "runas";
            psi.CreateNoWindow = false;
            psi.WindowStyle = ProcessWindowStyle.Hidden;
            psi.UseShellExecute = false;
            Process.Start(psi).WaitForExit();//删除urlacl
            psi = new ProcessStartInfo("netsh", args);
            psi.Verb = "runas";
            psi.CreateNoWindow = false;
            psi.WindowStyle = ProcessWindowStyle.Hidden;
            psi.UseShellExecute = false;
            Process.Start(psi).WaitForExit();//添加urlacl
        }

        /// <summary>
        /// 开启监听 '{"id":1,"data":"yourdata"}'
        /// </summary>
        /// <param name="listener"></param>
        private async void startListen(System.Net.HttpListener listener)
        {
            HttpListenerContext context = await listener.GetContextAsync();
            HttpListenerRequest request = context.Request;
            HttpListenerResponse response = context.Response;
            response.AppendHeader("Access-Control-Allow-Origin", "*");
            string rsp = "error";

            try
            {
                string order_uid = string.Empty;
                string aoid = string.Empty;
                string order_id = string.Empty;
                string price = string.Empty;
                string sign = string.Empty;
                string pay_price = string.Empty;


                using (StreamReader reader = new StreamReader(request.InputStream, Encoding.UTF8))
                {
                    string body = reader.ReadToEnd();
                    LogSys.InfoModule(LogTitle, $"body={body}");
                    string receivedData = HttpUtility.UrlDecode(body, Encoding.UTF8);
                    byte[] byte1= HttpUtility.UrlDecodeToBytes(body,Encoding.Default);
                    LogSys.InfoModule(LogTitle, $"获取Http的post数据. body={receivedData}");
                    string dencrypt = body; //AESHelp.AESDecrypt(body, AESHelp.KEY);
                    LogSys.InfoModule(LogTitle, dencrypt);
                    var headers = response.Headers;
                    headers.AllKeys.ToList().ForEach(i =>
                    {
                        LogSys.Warn("header key:" + i);
                    });

                    string data = string.Empty;
                    string msg = string.Empty;

                    JObject obj = JObject.Parse(dencrypt);
                    JToken msgToken = null;
                    JToken dataToken = null;

                    obj.TryGetValue("id", out msgToken);
                    obj.TryGetValue("data", out dataToken);

                    if (msgToken != null)
                        msg = msgToken.ToString();
                    if (dataToken != null)
                        data = dataToken.ToString();

                    HttpMsgId msgId = (HttpMsgId)int.Parse(msg);

                    switch (msgId)
                    {
                        //case HttpMsgId.Register:
                        //    rsp =  HttpResponseHelper.ReqUserRegister(Data);
                        //    break;
                        //case HttpMsgId.Login:
                        //    rsp = HttpListenerHandler.ReqUserLogin(data);
                        //    break;
                        //case HttpMsgId.Test:
                        //    rsp = HttpListenerHandler.ReqTestConnect(data);
                        //    break;
                        //case HttpMsgId.UploadGif:
                        //    rsp = await HttpListenerHandler.ReqUploadGif(data);
                        //    break;
                        //case HttpMsgId.GetOnlinePNums:
                        //    rsp = HttpListenerHandler.ReqOnlinePNums(data);
                        //    break;
                        //case HttpMsgId.GetNotice:
                        //rsp = HttpListenerHandler.RspNotice(data);
                        //    break;
                        //case HttpMsgId.GetEjectNotice:
                        //    rsp = HttpListenerHandler.RspEjectNotice(data);
                        //    break;
                        //case HttpMsgId.SetPlayerState:
                        //    rsp = HttpListenerHandler.RspSetPlayerState(data);
                        //    break;
                        case HttpMsgId.Ama:
                            stCode c = new stCode();
                            c.code = 0;
                            rsp = JObject.FromObject(c).ToString();
                            break;
                    }
                }
                //LogSys.InfoModule(HttpResponseLogName, $"获取Http的post返回的数据. rsp={rsp}");

            }
            catch (Exception e)
            {
                LogSys.Error($"httpListener throw exception : {e}");
            }
            finally
            {
                using (StreamWriter writer = new StreamWriter(response.OutputStream, Encoding.UTF8))
                {
                    string wStr = JSONUtil.JsonCompress(rsp);//AESHelp.AESEncrypt(rsp, AESHelp.KEY);
                    writer.Write(wStr);
                    writer.Close();
                    response.Close();
                }

                startListen(listener);
            }
        }

    }
}
